import redis
import logging
import json

from .preprocess import preprocess_file, preprocess_browse


logger = logging.getLogger(__name__)


def run_daemon(config, host, port, listen_queue, listen_md_queue, write_queue):
    """ Run the preprocessing daemon, listening on a redis queue
        for files to be preprocessed. After preprocessing the filename
        of the preprocessed files will be pushed to the output queue.
    """
    # initialize the queue client
    client = redis.Redis(
        host=host, port=port, charset="utf-8", decode_responses=True
    )
    logger.debug("waiting for redis queue '%s'..." % listen_queue)
    while True:
        # fetch an item from the queue to be preprocessed
        value, queue = client.brpop([listen_queue, listen_md_queue])
        filenames = []
        # start the preprocessing on that file
        if queue == listen_queue:
            filenames.append(preprocess_file(config, value))
        elif queue == listen_md_queue:
            browse_report_data = json.loads(value)
            browse_type = browse_report_data['browse_type']
            for browse in browse_report_data['browses']:
                filenames.append(preprocess_browse(config, browse_type, browse_report_data, browse))
        # TODO: convert to string, list, ....
        for item in filenames:
            client.lpush(write_queue, item)
